(******************************************************************************
 *                                                                            *
 *  Project: Adressbuch                                                       *
 *           Verwalten von Objekten in einer Container-Klasse                 *
 *  File   : AddressBookCls                                                   *
 *           Adressbuch-Klasse, Container-Klasse TContactList,                *
 *           Object-Klasse TContact                                           *
 *                                                                            *
 *  Copyright (c) Michael Puff  http://www.michael-puff.de                    *
 *                                                                            *
 ******************************************************************************)

 unit AddressBookCls;

interface

uses
  SysUtils, Classes;

type
  TContact = class(TObject)
  private
    FFirstName: String;
    FLastName: String;
    function GetFirstName: String;
    procedure SetFirstName(const Value: String);
    function GetLastName: String;
    procedure SetLastName(const Value: String);
  public
    property FirstName: String read GetFirstName write SetFirstName;
    property LastName: String read GetLastName write SetLastName;
  end;

  TContactList = class(TObject)
  private
    FInnerList: TList;
    function GetItem(Index: Integer): TContact;
    procedure SetItem(Index: Integer; Contact: TContact);
    function GetCount: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(Contact: TContact);
    procedure Delete(Index: Integer);
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TContact read GetItem write SetItem;
  end;

  TAddressBook = class(TObject)
  private
     FContacts: TContactList;
  public
    constructor Create;
    destructor destroy; override;
    property Contacts: TContactList read FContacts;
  end;

implementation

resourcestring
  rsObjectIsNil = 'Object is nil';

{ TContact }

// Getter

function TContact.GetFirstName: String;
begin
  Result := FFirstName;
end;

function TContact.GetLastName: String;
begin
  Result := FLastName;
end;

// Setter

procedure TContact.SetFirstName(const Value: String);
begin
  FFirstName := Value;
end;

procedure TContact.SetLastName(const Value: String);
begin
  FLastName := Value;
end;


{ TContactList }

// neu definierte Methode Add von TList

procedure TContactList.Add(Contact: TContact);
begin
  FInnerList.Add(Contact);
end;

function TContactList.GetCount: Integer;
begin
  Result := FInnerList.Count;
end;

procedure TContactList.Delete(Index: Integer);
begin
  // destroy object
  TObject(FInnerList.Items[Index]).Free;
  // delete object from the list
  FinnerList.Delete(Index);
end;

constructor TContactList.Create;
begin
  FInnerList := TList.Create;
end;

// berschriebener Destruktor
// Freigeben der Objekte in der Liste

destructor TContactList.Destroy;
var
  i                 : Integer;
begin
  if FInnerList.Count > 0 then
  begin
    for i := FInnerList.Count - 1 downto 0 do
    begin
      TObject(FInnerList.Items[i]).Free;
    end;
  end;
  FInnerList.Free;
  inherited;
end;

// Getter fr Item-Property

function TContactList.GetItem(Index: Integer): TContact;
begin
  Result := TContact(FInnerList.Items[Index]);
end;

// Setter fr Item-Property

procedure TContactList.SetItem(Index: Integer; Contact: TContact);
begin
  if Assigned(Contact) then
    FInnerList.Items[Index] := Contact
end;


{ TAddressBook }

constructor TAddressBook.Create;
begin
  inherited;
  FContacts := TContactList.Create;
end;

destructor TAddressBook.destroy;
begin
  FContacts.Free;
  inherited;
end;

end.

